--[[
AuraToChat, World of Warcraft addon that output buff gains to the in-game
ChatFrame.
Copyright (C) 2007 Rabbit

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
]]

-----------------------------------------------------------------------
-- AuraToChat

-----------------------------------------------------------------------
-- Locale

local PRINT_STRING = "%s on %s."
local ADD = "Add"
local ADD_DESC = "Adds a buff or debuff."
local REMOVE = "Remove"
local REMOVE_DESC = "Removes a buff or debuff."
local LIST = "List"
local LIST_DESC = "Lists the registered auras."
local COLOR = "Color"
local COLOR_DESC = "Sets the base color to use when calculating the aura colors."
local AURA_ADDED = "%s added."
local AURA_REMOVED = "%s removed."
local USAGE = "<aura name>"
local FIRST_USE = "AuraToChat has not been configured yet, you need to add auras to watch with the /atc add command."
do
	local locale = GetLocale()
	if locale == "koKR" then
		-- etc
	end
end

-----------------------------------------------------------------------
-- Utility functions and tables

-- Keyed table, key = spell name, value = color
local spellToColoredSpell = {}

local hexColors = {
	WTF = "|cffa0a0a0"
}
for k, v in pairs(RAID_CLASS_COLORS) do
	hexColors[k] = "|cff" .. string.format("%02x%02x%02x", v.r * 255, v.g * 255, v.b * 255)
end
local coloredName = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local class = select(2, UnitClass(key)) or "WTF"
		self[key] = hexColors[class]  .. key .. "|r"
		return self[key]
	end
})

local function findRGB(q, q2, hue)
	if hue > 360 then hue = hue - 360 end
	if hue < 0 then hue = hue + 360 end

	if hue < 60 then return q + (q2 - q) * hue / 60
	elseif hue < 180 then return q2
	elseif hue < 240 then return q + (q2 - q) * (240 - hue) / 60
	else return q
	end
end

local function hslToRGB(h, s, l)
	local l2, s2 = l / 100, s / 100

	local p2 = l2 + s2 - (l2 * s2)
	if l2 <= 0.5 then p2 = l2 * (1 + s2) end

	local p1 = 2 * l2 - p2

	local r, g, b = l2, l2, l2
	if s2 ~= 0 then
		r = findRGB(p1, p2, h + 120)
		g = findRGB(p1, p2, h)
		b = findRGB(p1, p2, h - 120)
	end
	r = math.ceil(r * 255)
	g = math.ceil(g * 255)
	b = math.ceil(b * 255)
	return r, g, b
end

local function rgbToHSL(r, g, b)
	local r2, g2, b2 = r / 51 * 0.2, g / 51 * 0.2, b / 51 * 0.2
	local max, min = 0, 0
	if r2 >= g2 then max = r2
	else max = g2 end
	if b2 > max then max = b2 end
	if r2 <= g2 then min = r2
	else min = g2 end
	if b2 < min then min = b2 end
	local l = (max + min) / 2
	local s, h = 0, 0
	if max ~= min then
		if l < .5 then
			s = (max - min) / (max + min)
		else
			s = (max - min) / (2 - max - min)
		end
		if r2 == max then h = (g2 - b2) / (max - min) end
		if g2 == max then h = (2 + (g2 - r2)) / (max - min) end
		if b2 == max then h = (4 + (r2 - g2)) / (max - min) end
	end
	h = math.ceil(h * 60)
	if h < 0 then h = h + 360
	elseif h >= 360 then h = h - 360 end
	s = math.ceil(s * 100)
	l = math.ceil(l * 100)
	return h, s, l
end

local function generateColors(r, g, b, auras)
	for k in pairs(spellToColoredSpell) do
		spellToColoredSpell[k] = nil
	end
	local h, s, l = rgbToHSL(r, g, b)
	for i = 1, #auras do
		local r, g, b = hslToRGB(h, s, l)
		spellToColoredSpell[auras[i]] = "|cff" .. string.format("%02x%02x%02x", r, g, b) .. auras[i] .. "|r"
		h = h + 30
		if h > 360 then h = h - 360 end
	end
end

-----------------------------------------------------------------------
-- Addon

AuraToChat = AceLibrary("AceAddon-2.0"):new(
	"AceEvent-2.0",
	"AceConsole-2.0",
	"AceDB-2.0",
	"Sink-1.0"
)
local addon = AuraToChat

function addon:OnInitialize()
	self:RegisterDB("AuraToChatDB")
	self:RegisterDefaults("profile", {
		sink10OutputSink = "ChatFrame",
		Auras = {},
		Color = {
			r = 255,
			g = 204,
			b = 153,
		},
	})
	self:RegisterChatCommand("/atc", {
		type = "group",
		args = {
			add = {
				type = "text",
				name = ADD,
				desc = ADD_DESC,
				get = false,
				set = function(aura)
					table.insert(self.db.profile.Auras, aura)
					generateColors(self.db.profile.Color.r, self.db.profile.Color.g, self.db.profile.Color.b, self.db.profile.Auras)
					self:Print(AURA_ADDED:format(spellToColoredSpell[aura]))
				end,
				usage = USAGE,
			},
			remove = {
				type = "text",
				name = REMOVE,
				desc = REMOVE_DESC,
				get = false,
				set = function(aura)
					for i, v in ipairs(self.db.profile.Auras) do
						if v == aura then
							local x = table.remove(self.db.profile.Auras, i)
							self:Print(AURA_REMOVED:format(spellToColoredSpell[x]))
							break
						end
					end
					generateColors(self.db.profile.Color.r, self.db.profile.Color.g, self.db.profile.Color.b, self.db.profile.Auras)
				end,
				usage = USAGE,
			},
			list = {
				type = "execute",
				name = LIST,
				desc = LIST_DESC,
				func = "Test",
			},
			color = {
				type = "color",
				name = COLOR,
				desc = COLOR_DESC,
				get = function()
					return self.db.profile.Color.r / 255, self.db.profile.Color.g / 255, self.db.profile.Color.b / 255
				end,
				set = function(r, g, b)
					self.db.profile.Color.r = r * 255
					self.db.profile.Color.g = g * 255
					self.db.profile.Color.b = b * 255
					self:ScheduleEvent(generateColors, 5, self.db.profile.Color.r, self.db.profile.Color.g, self.db.profile.Color.b, self.db.profile.Auras)
				end,
			},
		},
	}, "AURATOCHAT")

end

function addon:OnEnable(first)
	if #self.db.profile.Auras == 0 then
		self:Print(FIRST_USE)
	elseif first then
		generateColors(self.db.profile.Color.r, self.db.profile.Color.g, self.db.profile.Color.b, self.db.profile.Auras)
	end

	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("ADDON_LOADED")
end

local group = 0x007
local function shouldAccept(source)
	return bit.band(source, group) ~= 0
end
if COMBATLOG_OBJECT_AFFILIATION_MINE then
	group = COMBATLOG_OBJECT_AFFILIATION_MINE + COMBATLOG_OBJECT_AFFILIATION_PARTY + COMBATLOG_OBJECT_AFFILIATION_RAID
end
function addon:ADDON_LOADED(addon)
	if addon == "Blizzard_CombatLog" then
		group = COMBATLOG_OBJECT_AFFILIATION_MINE + COMBATLOG_OBJECT_AFFILIATION_PARTY + COMBATLOG_OBJECT_AFFILIATION_RAID
	end
end

local auras = setmetatable({}, {__index = function(t, k) t[k] = {}; return t[k] end})
local function auraLost(unitName, name)
	if not unitName or not spellToColoredSpell[name] then return end
	if not UnitInParty(unitName) and not UnitInRaid(unitName) then return end
	auras[unitName][name] = nil
end
local function aura(unitName, name)
	if not unitName or not spellToColoredSpell[name] then return end
	if not UnitInParty(unitName) and not UnitInRaid(unitName) then return end
	if auras[unitName][name] then return end
	auras[unitName][name] = true
	addon:Pour(PRINT_STRING:format(spellToColoredSpell[name], coloredName[unitName]))
end

function addon:COMBAT_LOG_EVENT_UNFILTERED(timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId)
	if eventType ~= "SPELL_AURA_APPLIED" and eventType ~= "SPELL_AURA_REMOVED" then return end
	if not shouldAccept(dstFlags) then return end
	local spellName = GetSpellInfo(spellId)
	if not spellName then return end
	if eventType == "SPELL_AURA_APPLIED" then
		aura(dstName, spellName)
	elseif eventType == "SPELL_AURA_REMOVED" then
		auraLost(dstName, spellName)
	end
end

local playerName = UnitName("player")
function addon:Test()
	for k, v in pairs(spellToColoredSpell) do
		aura(playerName, k)
		auraLost(playerName, k)
	end
end

